import { parse } from 'himalaya/src/index.js';

export function getConfig(cfg) {
	cfg.name = 'Metallum (Unsynced)';
	cfg.version = '0.2';
	cfg.author = 'TT';
	cfg.useRawMeta = false;
}

export function getLyrics(meta, man) {
	const Clean = (text) => text
		.replace(/\(.*\)|{.*}|\[.*\]|【.*】/g, '').normalize().trim().toLowerCase()
		.replace(/[^a-z0-9\- ]/g, '')
		.replace(/@/g, 'at')
		.replace(/&/g, 'and');

	const artist = Clean(meta.artist).replace(/^the /, '');
	const album = Clean(meta.album);
	const title = Clean(meta.title);

	if (!artist || !album || !title) return;

	// *  1. ARTIST SEARCH * //
	searchArtist(artist, (bands) => {
		// *  2. ALBUM SEARCH * //
		searchAlbum(album, (albums) => {
			const validPairs = albums.reduce((acc, albumItem) => {
				if (!albumItem.band) return acc;
				const band = bands.find(b => b.id === albumItem.band.id);
				return band ? [...acc, { band, album: albumItem }] : acc;
			}, []);

			for (const { band, album: albumData } of validPairs) {
				getAlbum(albumData.id, (fullAlbumData) => {
					if (!fullAlbumData.songs || !fullAlbumData.songs.length) return;

					// *  3. TRACK SEARCH * //
					const song = fullAlbumData.songs.find(s => {
						const cleanedName = Clean(s.name);
						return cleanedName.includes(title) || title.includes(cleanedName);
					});

					if (!song && !song.lyrics) return;

					const lyricMeta = man.createLyric();
					lyricMeta.title = meta.title;
					lyricMeta.artist = meta.artist;
					lyricMeta.lyricText = parseLyrics(song.lyrics);
					lyricMeta.location = `https://metal-api.dev/albums/${albumData.id}`;
					man.addLyric(lyricMeta);
				});
			}
		});
	});
}

function searchArtist(query, callback) {
	const url = `https://metal-api.dev/search/bands/name/${encodeURIComponent(query)}`;

	request({
		url,
		headers: {
		'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36',
		'Accept': 'application/json'
		}
	}, (err, res, body) => {
		if (err || res.statusCode !== 200) {
			return callback(null, []);
		}
		const data = JSON.parse(body);
		callback(Array.isArray(data) ? data : []);
	});
}

function searchAlbum(query, callback) {
	const url = `https://metal-api.dev/search/albums/title/${encodeURIComponent(query)}`;

	request({
		url,
		headers: {
		'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36',
		'Accept': 'application/json'
		}
	}, (err, res, body) => {
		if (err || res.statusCode !== 200) {
			return callback(null, []);
		}
		const data = JSON.parse(body);
		callback(Array.isArray(data) ? data : []);
	});
}

function getAlbum(id, callback) {
	const url = `https://metal-api.dev/albums/${id}`;

	request({ url }, (err, res, body) => {
		if (err || res.statusCode !== 200) {
			return callback(null, { songs: [] });
		}
		const data = JSON.parse(body);
		callback(data);
	});
}

function parseLyrics(lyricText) {
	const Clean = (rawString) => rawString.trim()
		.replace(/&#x([0-9a-f]+);/gi, (_, code) => String.fromCharCode(parseInt(code, 16))) // HTML characters decode
		.replace(/&amp(;|)/gi, '&')
		.replace(/&gt(;|)/gi, '>')
		.replace(/&lt(;|)/gi, '<')
		.replace(/&nbsp(;|)/gi, '')
		.replace(/&quot(;|)/gi, '"')
		.replace(/<br>/gi, '')
		.replace(/\uFF1A/gi, ':')
		.replace(/\uFF08/gi, '(')
		.replace(/\uFF09/gi, ')')
		.replace(/\u00E2\u20AC\u2122|\u2019|\uFF07|[\u0060\u00B4]|â€™(;|)|â€˜(;|)|&apos(;|)|&#39(;|)|(&#(?:039|8216|8217|8220|8221|8222|8223|x27);)/gi, "'") // Apostrophe variants
		.replace(/[\u2000-\u200F\u2028-\u202F\u205F-\u206F\u3000\uFEFF]/gi, ' ') // Whitespace variants
		.replace(/<br \/>/gi, ''); // Metallum formatting

	return Clean(lyricText);
}
